/*
 * BBXKeys.c
 *
 * By Jamie McCarthy, Jan 93.  This is not public domain;  it is
 *  Copyright 1993 by Jamie R. McCarthy.  All rights reserved.
 * But you are free (in both the legal and the financial sense)
 * to distribute this program, and its source, to anyone you
 * like.  Yes, Virginia, this is FreeWare.
 *
 * If you have any questions or comments or anything, you can
 * reach me at "j.mccarthy" on AppleLink, or at the Internet
 * address "k044477@kzoo.edu".
 *
 * Apologies for the paucity of comments.
 *
 */



/******************************/

#include <SetupA4.h>
#include "ExternalInterface.h"
#include "DialogUtilities.h"

/******************************/

typedef struct {
	Str255 theDefTextStr;
} bbxkPrefs, *bbxkPrefsPtr;

/******************************/

enum {
	kMainID = 128,
	kAboutID,
	kErrorID,
	kDefaultID = 128
} ;

enum {
	kOK = 1,
	kCancel,
	kAbout,
	kEditText
} ;

enum {
	kAboutText = 2
} ;

enum {
	kErrBadVersion = 1,
	kErrNoBackslash,
	kErrNoExtensionFound,
	kErrBadCmdKey,
	kErrNoMenuFound
} ;

/******************************/

ExternalCallbackBlock *gCallbacks;
short gErrNum;
short gErrLine;

Boolean pStringsEqual(unsigned char *pStr1, unsigned char *pStr2);

void setupPrefs(void);
void setPrefsText(Str255 theDefaultStr);

Boolean doDialog(void);

void doAssign(void);

void reportErr(void);

/******************************/



pascal void main(ExternalCallbackBlock *callbacks, WindowPtr w)
{
	RememberA0();
	SetUpA4();
	
	gCallbacks = callbacks;
	gErrNum = 0;
	gErrLine = 0;
	
	if (gCallbacks->version < 2) {
		gErrNum = kErrBadVersion;
	} else {
		setupPrefs();
		if (doDialog()) {
			doAssign();
		}
	}
	
	if (gErrNum > 0) {
		reportErr();
	}
	
	RestoreA4();
}



/******************************/



void setupPrefs(void)
{
	Str255 theTextStr;
	Handle theDefaultTextHndl;
	short theTextLen;
	short realTextLen;
	
	theDefaultTextHndl = NewHandle(sizeof(bbxkPrefs));
	HLock(theDefaultTextHndl);
	gCallbacks->GetPreference('BBXK', sizeof(bbxkPrefs),
		*theDefaultTextHndl, &realTextLen);
	
	if (realTextLen < sizeof(bbxkPrefs)) {
		
		Handle theStrHndl;
		theStrHndl = GetResource('STR ', kDefaultID);
		HLock(theStrHndl);
		setPrefsText((unsigned char *) *theStrHndl);
		ReleaseResource(theStrHndl);
		
	}
	
	DisposHandle(theDefaultTextHndl);
}



	void normalizeStringForCRs(unsigned char *theString);
	void normalizeStringForCRs(register unsigned char *theString)
	{
		while (theString[0] > 0 && theString[theString[0]] == '\r') {
				// Strip a <CR> off the end of the string.
			--theString[0];
		}
		if (theString[0] > 0) {
				// Append a <CR>, but only if the string's not empty.
			theString[++theString[0]] = '\r';
		}
	}
	
void setPrefsText(Str255 theDefaultString)
{
	short realTextLen;
	normalizeStringForCRs(theDefaultString);
	gCallbacks->SetPreference('BBXK', sizeof(bbxkPrefs), theDefaultString, &realTextLen);
}



/******************************/



Boolean pStringsEqual(unsigned char *pStr1, unsigned char *pStr2)
{
	short nChars = *pStr1;
	while ( nChars && (pStr1[nChars] == pStr2[nChars]) ) --nChars;
	return (nChars == 0);
}



	pascal void drawAboutText(WindowPtr theWindPtr, short wItem);
	pascal void drawAboutText(WindowPtr theWindPtr, short wItem)
	{
		short theItemType;
		Handle theItemHndl;
		Rect theItemBox;
		
		Handle theTextHndl;
		long theTextLength;
		
		GetDItem((DialogPtr) theWindPtr, wItem, &theItemType, &theItemHndl, &theItemBox);
		
		theTextHndl = GetResource('TEXT', kAboutID);
		theTextLength = GetHandleSize(theTextHndl);
		
		TextFont(geneva);
		TextFace(normal);
		TextMode(srcOr);
		TextSize(9);
		
		HLock(theTextHndl); // necessary?
		TextBox(*theTextHndl, theTextLength, &theItemBox, teJustLeft);
		HUnlock(theTextHndl);
	}
	
void doAbout(void);
void doAbout(void)
{
	GrafPtr oldPort;
	DialogPtr theDlgPtr;
	short wItem;
	
	short theItemType;
	Handle theItemHndl;
	Rect theItemBox;
	
	Boolean done;
	
	GetPort(&oldPort);
	theDlgPtr = GetNewDialog(kAboutID, NULL, (WindowPtr) -1);
	SetPort(theDlgPtr);
	
	GetDItem(theDlgPtr, kAboutText, &theItemType, &theItemHndl, &theItemBox);
	theItemHndl = (Handle) drawAboutText;
	SetDItem(theDlgPtr, kAboutText, theItemType, theItemHndl, &theItemBox);
	
	done = FALSE;
	while (!done) {
		ModalDialog(gCallbacks->StandardFilter, &wItem);
		if (wItem == kOK) {
			done = TRUE;
		}
	}
	
	DisposDialog(theDlgPtr);
	SetPort(oldPort);
}



void doUpdateDialog(DialogPtr theDlgPtr);
void doUpdateDialog(DialogPtr theDlgPtr)
{
	Str255 theTextStr;
	
	ReadStrItem(theDlgPtr, kEditText, theTextStr);
	if (pStringsEqual("\pHerzliche Gre, Irene!", theTextStr)) {
		
		Str255 theTextStr;
		Handle theDefaultTextHndl;
		short theTextLen;
		short realTextLen;
		
		gCallbacks->GetPreference('BBXK', 256, &theTextStr, &realTextLen);
		
		normalizeStringForCRs(theTextStr);
		
		SetStrItem(theDlgPtr, kEditText, theTextStr);
		SelIText(theDlgPtr, kEditText, theTextStr[0], theTextStr[0]);
		
	}
}



pascal Boolean bbxkAlrtFilter(DialogPtr theDlgPtr,
	EventRecord *theEventRec, short *itemHit);
pascal Boolean bbxkAlrtFilter(DialogPtr theDlgPtr,
	EventRecord *theEventRec, short *itemHit)
{
	if ( theEventRec->what == keyDown
		&& (theEventRec->message&0x00FF) == '\r'
		&& (theEventRec->modifiers&(cmdKey|shiftKey|alphaLock|optionKey)) == 0 ) {
			// We want to handle <CR>s ourselves.
		return FALSE;
	} else {
			// If it's not a <CR>, let BBEdit decide what to do.
		return gCallbacks->StandardFilter(theDlgPtr, theEventRec, itemHit);
	}
}



Boolean doDialog(void)
{
	GrafPtr oldPort;
	DialogPtr theDlgPtr;
	short wItem;
	Boolean done;
	
	GetPort(&oldPort);
	theDlgPtr = gCallbacks->CenterDialog(kMainID);
	SetPort(theDlgPtr);
	
	done = FALSE;
	while (!done) {
		
		doUpdateDialog(theDlgPtr);
		
		ModalDialog(bbxkAlrtFilter, &wItem);
		
		switch (wItem) {
			
			case kOK:			done = TRUE;										break;
			case kCancel:		done = TRUE;										break;
			case kAbout:		doAbout();											break;
			
			default:																		break;
		}
		
	}
	
	if (wItem == kOK) {
		Str255 theTextStr;
		ReadStrItem(theDlgPtr, kEditText, theTextStr);
		setPrefsText(theTextStr);
	}
	
	DisposDialog(theDlgPtr);
	SetPort(oldPort);
	
	return (wItem == kOK);
}



/******************************/



	void getSubPStr(unsigned char *theStr, short startChar,
		unsigned char theDelimChar, unsigned char *subStr);
	void getSubPStr(register unsigned char *theStr, short startChar,
		register unsigned char theDelimChar, register unsigned char *subStr) 
	{
		register short cChar;
		register short nChars;
		
		nChars = theStr[0];
		cChar = startChar;
		while (cChar <= nChars && theStr[cChar] != theDelimChar) {
			subStr[cChar - startChar + 1] = theStr[cChar];
			++cChar;
		}
		if (theStr[cChar] != theDelimChar) {
			subStr[0] = 0;
		} else {
			subStr[0] = cChar - startChar;
		}
	}
	
	unsigned char *getKbdEquivCharPtr(unsigned char *theData, unsigned char *theName);
	unsigned char *getKbdEquivCharPtr(unsigned char *theData, unsigned char *theName)
	{
			/*
			 * In case something goes wrong and we're not pointing to a menu data
			 * structure, a few checks will be done.  It will be assumed, first,
			 * that there will be less than 50 items in the menu.  Second, that
			 * each item name is less than 60 characters long.
			 */
		short cNameLength;
		short cMenuItem;
		cNameLength = *theData;
		cMenuItem = 1;
		while (cMenuItem < 50							// We haven't gone too far, have we?
			&& cNameLength < 60							// The name's not too long, is it?
			&& cNameLength != 0							// Are we at the end of the menu?
			&& !pStringsEqual(theData, theName)) {	// We haven't found the item, have we?
			theData += cNameLength + 5;
			cNameLength = *theData;
			++cMenuItem;
		}
		if (cMenuItem >= 50 || cNameLength >= 60		// Something went wrong.
			|| cNameLength == 0) {							// The item isn't in the menu.
			return NULL;
		} else {
			return theData + cNameLength + 2;
		}
	}
	
void processMenuData(unsigned char *theData);
void processMenuData(unsigned char *theData)
{
	Str255 thePrefsStr;
	short realLength;
	Str255 cExtensionName;
	short startChar;
	short cLine;
	
	cLine = 1;
	startChar = 1;
	
	if (getKbdEquivCharPtr(theData, "\pBBXKeys") == NULL) {
			/*
			 * There's no keyboard equivalent character for "BBXKeys", so
			 * that is not an item in the menu.  That means we've got the
			 * wrong menu.  Waitaminute!  Something's wrong.  Beg out.
			 */
		gErrNum = kErrNoMenuFound;
		goto @errorInProcessMenuData;
	}
	
	gCallbacks->GetPreference('BBXK', sizeof(bbxkPrefs), thePrefsStr, &realLength);
	
		/*
		 * Do nothing for a zero-length string.
		 */
	if (thePrefsStr[0] == 0) {
		return;
	}
	
	while (getSubPStr(thePrefsStr, startChar, '\\', cExtensionName),
		cExtensionName[0] != 0) {
		
		unsigned char *theKbdEquivCharPtr;
		startChar += cExtensionName[0] + 1;
		theKbdEquivCharPtr = getKbdEquivCharPtr(theData, cExtensionName);
		if (theKbdEquivCharPtr == NULL) {
			gErrNum = kErrNoExtensionFound;
			goto @errorInProcessMenuData;
		}
		getSubPStr(thePrefsStr, startChar, '\r', cExtensionName);
		if (cExtensionName[0] != 1) {
			gErrNum = kErrBadCmdKey;
			goto @errorInProcessMenuData;
		}
		*theKbdEquivCharPtr = cExtensionName[1];
		startChar += 2;
		
		++cLine;
	}
	
	if (startChar != thePrefsStr[0]+1) {
		gErrNum = kErrNoBackslash;
@errorInProcessMenuData:
		gErrLine = cLine;
	}
}



void doAssign(void)
{
	short cMenu, nMenus;
	Handle menuListHndl;
	
	menuListHndl = MenuList;
	nMenus = *(short*)*menuListHndl / 6;
	for (cMenu = 1; cMenu <= nMenus; ++cMenu) {
		MenuHandle cMenuHndl;
		cMenuHndl = * (MenuHandle*) (*menuListHndl + 6*cMenu);
		if ((**cMenuHndl).menuID == 1035) {
			short menuTitleLength;
			menuTitleLength = (**cMenuHndl).menuData[0];
			processMenuData( &(**cMenuHndl).menuData[menuTitleLength+1] );
			return ;
		}
	}
	
	gErrNum = kErrNoMenuFound;
}



/******************************/



void reportErr(void)
{
	Str15 theErrNumStr;
	Str15 theErrLineStr;
	Str255 theErrTextStr;
	NumToString(gErrNum, theErrNumStr);
	NumToString(gErrLine, theErrLineStr);
	GetIndString(theErrTextStr, kErrorID, gErrNum);
	ParamText(theErrNumStr, theErrLineStr, theErrTextStr, NULL);
	/* theClickedItem = */ StopAlert(kErrorID, gCallbacks->StandardFilter);
}

